@@ -1,8 +1,7 @@
 import copy
 import os
 import re
-import sys
-from typing import List
+from typing import List, Literal
 
 from clang.cindex import AccessSpecifier, Config, Cursor, CursorKind, TranslationUnit
 
@@ -14,8 +13,7 @@ logger = get_logger("parser")
 
 class Parser:
     """
-    CPPyGen はヘッダーを解析し、pybind11 用の関数を自動で作り出すコードジェネレー
-    タです。
+    Analyze C++ source files and Generate pybind11 C++ code.
     """
 
     def __init__(
@@ -24,12 +22,14 @@ class Parser:
         *,
         library_path: str | None = None,
         library_file: str | None = None,
+        verbose: bool = False,
     ):
-        self._funcitons: List[Function] = []
+        self._functions: List[Function] = []
         self._submodules: List[Submodule] = []
         self._structs_and_classes: List[StructOrClass] = []
         self._hpp_includes: List[str] = []
         self._namespace = namespace or "cppygen"
+        self.verbose = verbose
 
         if library_file != None and library_path != None:
             raise ValueError(f"Both library_path and library_file cannot be set.")
@@ -50,23 +50,25 @@ class Parser:
             flags = []
         if (cppygen_flags := os.environ.get("CPPYGEN_COMPILE_FLAGS", None)) is not None:
             flags.extend(cppygen_flags.split(" "))
-            print(cppygen_flags.split(" "))
         args = list(flags)
         name = filename
         return TranslationUnit.from_source(
             name,
             args,
             unsaved_files=[(name, source)],
-            options=TranslationUnit.PARSE_DETAILED_PROCESSING_RECORD,
+            options=TranslationUnit.PARSE_NONE,
         )
 
-    def _extract_functions(self, cu: Cursor, namespace: List[str], module_name: str):
-        """
-        cu 以下にある関数を抽出する
-        """
+    def _extract_functions(
+        self,
+        cu: Cursor,
+        namespace: List[str],
+        module_name: str,
+        mode: Literal["source"] | Literal["header"] = "source",
+    ):
         for i in list(cu.get_children()):
             i: Cursor
-            if i.kind == CursorKind.FUNCTION_DECL and i.is_definition():  # type: ignore
+            if i.kind == CursorKind.FUNCTION_DECL and (i.is_definition() or mode == "header"):  # type: ignore
                 func = Function()
                 func.set_return_type(i.result_type.spelling)
                 func.set_name(i.spelling, namespace)
@@ -74,25 +76,25 @@ class Parser:
 
                 # extract comment string
                 raw_comment = i.raw_comment or ""
-                pyname = re.search(r"pyname: *(.*)", raw_comment)
-                description = re.search(r"description: *(.*)", raw_comment)
+
+                pyname, description = _extract_comment_string(raw_comment)
+
                 if pyname is not None:
-                    func.pyname = pyname[1]
-                func.set_description(description and description[1] or "")
+                    func.pyname = pyname
+                func.set_description(description and description or "")
 
                 for j in list(i.get_children()):
                     j: Cursor
                     if j.kind == CursorKind.PARM_DECL:  # type: ignore
                         func.add_argument_type((j.spelling, j.type.spelling))
-                print("\t| Function  | " + func.signature())
-                self._funcitons.append(func)
+                if self.verbose:
+                    print("\t| Function  | " + func.signature())
+                if not func in self._functions:
+                    self._functions.append(func)
 
     def _extract_struct_and_class(
         self, cu: Cursor, namespace: List[str], module_name: str
     ):
-        """
-        cu 以下にある構造体を抽出する
-        """
         for i in list(cu.get_children()):
             i: Cursor
             if i.kind == CursorKind.STRUCT_DECL or i.kind == CursorKind.CLASS_DECL:  # type: ignore
@@ -116,14 +118,19 @@ class Parser:
                         for k in list(j.get_children()):
                             if k.kind == CursorKind.PARM_DECL:  # type: ignore
                                 args.append((k.spelling, k.type.spelling))
+                        (pyname, description) = _extract_comment_string(
+                            j.raw_comment or ""
+                        )
                         struct_or_class.add_member_func(
                             j.spelling,
+                            pyname,
                             j.result_type.spelling,
                             args,
-                            j.brief_comment or "",
+                            description or "",
                             j.access_specifier == AccessSpecifier.PRIVATE,  # type: ignore
                         )
-                print("\t| Class     | " + struct_or_class.signature())
+                if self.verbose:
+                    print("\t| Class     | " + struct_or_class.signature())
                 self._structs_and_classes.append(struct_or_class)
 
     def add_hpp_includes(self, hpp: str):
@@ -136,6 +143,7 @@ class Parser:
         lang: str = "cpp",
         flags=[],
         with_diagnostic=False,
+        mode: Literal["source"] | Literal["header"] = "source",
     ):
         tu: TranslationUnit = self._get_tu(source, filename, flags)
         if with_diagnostic:
@@ -152,12 +160,18 @@ class Parser:
         for i in list(root.get_children()):
             i: Cursor
 
-            # 再起的に関数を抽出する。
+            # Recursive Function
             def visit(x: Cursor, namespace: List[str], module_name: str):
-                if lang == "cpp":
-                    self._extract_functions(x, namespace, module_name)
-                elif lang == "hpp":  # ヘッダーでのみクラスを抽出する。
+                if mode == "source":
+                    if lang == "cpp":
+                        self._extract_functions(x, namespace, module_name)
+                    elif lang == "hpp":
+                        self._extract_struct_and_class(x, namespace, module_name)
+                elif mode == "header":
+                    self._extract_functions(x, namespace, module_name, mode="header")
                     self._extract_struct_and_class(x, namespace, module_name)
+                else:
+                    raise KeyError('Mode should be "source" or "header"')
                 for i in list(x.get_children()):
                     i: Cursor
                     namespace_in = copy.deepcopy(namespace)
@@ -167,24 +181,31 @@ class Parser:
                         submod.set_description(i.brief_comment or "")
                         submod.set_parent(copy.deepcopy(namespace_in))
                         if not submod in self._submodules:
-                            print(f"\t| Submodule | {submod.cpp_name}")
+                            if self.verbose:
+                                print(f"\t| Submodule | {submod.cpp_name}")
                             self._submodules.append(submod)
                         namespace_in.append(i.spelling)
                         visit(i, namespace_in, submod.cpp_name)
 
-            # トップレベルの namespace を探す
+            # Search top-level namespace
             if i.kind == CursorKind.NAMESPACE and i.spelling == self._namespace:  # type: ignore
                 visit(i, [self._namespace], self._namespace)
 
-    def parse_from_file(self, filename: str, lang: str = "cpp", flags=[]):
+    def parse_from_file(
+        self,
+        filename: str,
+        lang: str = "cpp",
+        flags=[],
+        mode: Literal["header"] | Literal["source"] = "source",
+    ):
         with open(filename, "r") as f:
             data = f.read()
-        self.parse(data, filename, lang, flags, with_diagnostic=True)
+        self.parse(data, filename, lang, flags, with_diagnostic=True, mode=mode)
 
     def to_decl_string(self):
         return (
             "/* Function Declarations Start */\n"
-            + "\n".join([i.to_decl_string() for i in self._funcitons] + [""])
+            + "\n".join([i.to_decl_string() for i in self._functions] + [""])
             + "/* Function Declarations End */\n\n"
         )
 
@@ -198,9 +219,20 @@ class Parser:
     def to_export_string(self):
         return (
             "\t/* Function Export Start */\n"
-            + "\n".join(["\t" + i.to_pybind_string() for i in self._funcitons] + [""])
+            + "\n".join(
+                [
+                    # overloaded function
+                    "\t" + i.to_pybind_string(overloaded=True)
+                    # Check if more than two function has same signature.
+                    if [j._full_name for j in self._functions].count(i._full_name) > 1
+                    # Non overloaded function
+                    else "\t" + i.to_pybind_string()
+                    for i in self._functions
+                ]
+                + [""]
+            )
             + "\t/* Function Export End */\n\n"
-            "\t/* Structs and Classes Export Start */\n"
+            + "\t/* Structs and Classes Export Start */\n"
             + "\n".join(
                 ["\t" + i.to_pybind_string() for i in self._structs_and_classes] + [""]
             )
@@ -270,3 +302,10 @@ class Parser:
             f"extern void CPPyGenExport(pybind11::module_ {self._namespace});\n\n"
             "}"
         )
+
+
+def _extract_comment_string(raw_comment: str) -> tuple[str | None, str | None]:
+    return (
+        (re.search(r"pyname: *(.*)", raw_comment) or [None, None])[1],
+        (re.search(r"description: *(.*)", raw_comment) or [None, None])[1],
+    )
